// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// Implementation of HSVSIFTExtractor.

#include <assert.h>
#include <stdio.h>
#include <vector>
#include <string>
#include "extractor/hsv-sift-extractor.h"
#include "interest_point.h"
#include "point_set/point-set.h"
#include "point_set/point.h"

using namespace std;
using namespace libpmk;

namespace libpmk_features {

void HSVSIFTExtractor::SetBinaryPath(const string& path) {
  sift_extractor_.SetBinaryPath(path);
}

void HSVSIFTExtractor::ConvertRGBToHSV(int R, int G, int B,
                                       int* H, int *S, int* V) {
  assert(H != NULL);
  assert(S != NULL);
  assert(V != NULL);
  int min = R, max = R;
  if (G < min) min = G;
  if (B < min) min = B;
  if (G > max) max = G;
  if (B > max) max = B;

  *V = max;
  if (max == 0) {
    *H = 0;
    *S = 0;
    return;
  }

  int delta = max - min;
  *S = (255 * (delta)) / *V;
  if (*S == 0) {
    *H = 0;
    return;
  }

  if (max == R) {
    *H = (43 * (G - B)) / delta;
  } else if (max == G) {
    *H = 85 + ((43 * (B - R)) / delta);
  } else {
    *H = 171 + ((43 * (R - G)) / delta);
  }
}

PointSet HSVSIFTExtractor::ExtractFeatures(const vector<InterestPoint>& points,
                                           const string& image) const {
  // Create four temp files. The first one is the image converted to ppm.
  // The other three are pgm files, each being one HSV channel.
  // Create three temp files, one for each HSV component.
  char cmd[4096];
  char ppm_file[1024];
  char h_file[1024];
  char s_file[1024];
  char v_file[1024];

  sprintf(ppm_file, "/tmp/p%d.XXXXXX", getpid());
  mkstemp(ppm_file);

  sprintf(h_file, "/tmp/h%d.pgm", getpid());
  sprintf(s_file, "/tmp/s%d.pgm", getpid());
  sprintf(v_file, "/tmp/v%d.pgm", getpid());

  // Do the conversion to ppm:
  sprintf(cmd, "./convert %s ppm:%s", image.c_str(), ppm_file);
  system(cmd);

  // Prepare the three other temp files for writing:
  FILE* h_output = fopen(h_file, "w");
  FILE* s_output = fopen(s_file, "w");
  FILE* v_output = fopen(v_file, "w");
  fprintf(h_output, "P2\n");
  fprintf(s_output, "P2\n");
  fprintf(v_output, "P2\n");

  // Read the ASCII PPM data:
  sprintf(cmd, "./pnmtoplainpnm %s", ppm_file);
  FILE *p = popen(cmd, "r");
  fscanf(p, "P3");
  int image_width, image_height, max_value;
  fscanf(p, "%d %d", &image_width, &image_height);
  fscanf(p, "%d", &max_value);
  assert(max_value == 255);

  fprintf(h_output, "%d %d\n255\n", image_width, image_height);
  fprintf(s_output, "%d %d\n255\n", image_width, image_height);
  fprintf(v_output, "%d %d\n255\n", image_width, image_height);

  for (int ii = 0; ii < image_width; ++ii) {
    for (int jj = 0; jj < image_height; ++jj) {
      int R, G, B;
      int H, S, V;
      fscanf(p, "%d %d %d", &R, &G, &B);
      ConvertRGBToHSV(R, G, B, &H, &S, &V);
      fprintf(h_output, "%d\n", (int)H);
      fprintf(s_output, "%d\n", (int)S);
      fprintf(v_output, "%d\n", (int)V);
    }
  }
  pclose(p);
  fclose(h_output);
  fclose(s_output);
  fclose(v_output);

  // Convert the pgms to raw format. ImageMagick automatically does this.
  sprintf(cmd, "./convert %s %s", h_file, h_file);
  system(cmd);
  sprintf(cmd, "./convert %s %s", s_file, s_file);
  system(cmd);
  sprintf(cmd, "./convert %s %s", v_file, v_file);
  system(cmd);


  PointSet h_sift(sift_extractor_.ExtractFeatures(points, h_file));
  PointSet s_sift(sift_extractor_.ExtractFeatures(points, s_file));
  PointSet v_sift(sift_extractor_.ExtractFeatures(points, v_file));
  unlink(ppm_file);
  unlink(h_file);
  unlink(s_file);
  unlink(v_file);

  // Resulting descriptor is 3 * 128-D (three SIFT descriptors):
  assert(h_sift.size() == s_sift.size());
  assert(s_sift.size() == v_sift.size());

  PointSet output(3 * 128);
  for (int ii = 0; ii < h_sift.size(); ++ii) {
    const Point& h_point = h_sift.point(ii);
    const Point& s_point = s_sift.point(ii);
    const Point& v_point = v_sift.point(ii);

    // If this point is not bogus, add it to the output point set.
    if (h_point.feature(0) > 0 &&
        s_point.feature(0) > 0 &&
        v_point.feature(0) > 0) {
      Point* new_point = output.add_point();
      for (int ii = 0; ii < 128; ++ii) {
        new_point->set_feature(ii, h_point.feature(ii));
      }

      for (int ii = 0; ii < 128; ++ii) {
        new_point->set_feature(128 + ii, s_point.feature(ii));
      }

      for (int ii = 0; ii < 128; ++ii) {
        new_point->set_feature(256 + ii, v_point.feature(ii));
      }
    }
  }

  return output;
}

}  // namespace libpmk_features
